"use strict";

this.name    = "AI Autotrade"; // The original and genuine automatic trading system.
this.version = "2.12.4";
this.$mode   = ["advice", 1, 4]; // Default settings for new users.

/* ====================================================================================
			MESSAGES FOR DISPLAY

	These are the only text strings defined in this script. See missiontext.plist 
	file for the purchase & sale recommendations and the starting messages.
======================================================================================= */

this.$startMessage   = ["", "Autotrade", ""]; // See missiontext.plist file.
this.$flummoxMessage = ["Trading Assistant is flummoxed. Human trading decisions will be required.", "Flummoxed.", ""];
this.$noMoreMessage  = ["No further recommendations for you, boss.", "", ""];
this.$finishMessage  = ["Fully automatic trading has finished.", "Finished.", ""];

/* ====================================================================================
			TIMINGS
======================================================================================= */

this.$firstWarning   = 12; // Seconds until this.$startMessage is displayed.
this.$firstAdvice    = 15; // Seconds until first recommendation is displayed.
this.$firstTrade     = 22; // Seconds until first trade is executed.
this.$tradePeriod    = 9;  // Seconds between consecutive trades.

/* ====================================================================================
			TABLES OF COMMODITY NAMES AND EXPECTED PRICES
======================================================================================= */

this.$commodities    = ["food", "textiles", "radioactives", "slaves", "liquor_wines", "luxuries", "narcotics", "computers", "machinery", "alloys", "firearms", "furs", "minerals", "gold", "platinum", "gem_stones", "alien_items"];
this.$expectedPrices = [50,72,232,152,292,902,510,818,566,388,692,705,120,388,718,196,436]; // deci-credits

/* ====================================================================================
			STORED SETTINGS
======================================================================================= */

    this.startUp = function () {
        "use strict";
        if (missionVariables.tradingAI_autotrade) {
            this.$mode = JSON.parse(missionVariables.tradingAI_autotrade);
        }
        if (!this.$mode[3]) {
            this.$mode[3] = {}
            this.$mode[3].buyBelow  = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]; // deci-credits
            this.$mode[3].sellAbove = [1020,1020,1020,1020,1020,1020,1020,1020,1020,1020,1020,1020,1020,1020,1020,1020,1020]; // deci-credits
        }
        this._setEthics(this.$mode[2]);
        delete this.startUp;
        if (!worldScripts['AI Trading Assistant']) {
            this._flummoxResponse = this._flummoxAlternative;
        }
        delete this._flummoxAlternative;
    }

    this.playerWillSaveGame = function () {
        "use strict";
        missionVariables.tradingAI_autotrade = JSON.stringify(this.$mode);
    }

/* ====================================================================================
			TRADING ETHICS
======================================================================================= */

    this.shipDockedWithStation_UNSET = function (station) {
        "use strict";
        if (station.isMainStation) {
		this.$expectedPrices[3]  = 20;  // slaves
		this.$expectedPrices[6]  = 4;   // narcotics
		this.$expectedPrices[10] = 496; // firearms
//		this.$startMessage[0] = expandMissionText("autotrade_ethical_start");
		this.$startMessage[0] = expandMissionText("autotrade_ethical_start", {autotrade_countdown: (this.$firstTrade - this.$firstWarning).toFixed()});
        } else { // Rock hermit or OXP station...
		this.$expectedPrices[3]  = 152; // slaves
		this.$expectedPrices[6]  = 510; // narcotics
		this.$expectedPrices[10] = 692; // firearms
//		this.$startMessage[0] = expandMissionText("autotrade_ruthless_start");
		this.$startMessage[0] = expandMissionText("autotrade_ruthless_start", {autotrade_countdown: (this.$firstTrade - this.$firstWarning).toFixed()});
        }
    }

    this._setEthics = function (mode) {
        "use strict";
        if (mode !== 2) {
            delete this.shipDockedWithStation;
        }
        if (mode !== 4) {
            delete this.playerBoughtCargo;
            delete this.playerSoldCargo;
        }
        switch (mode) {
          case 1:
            this.$expectedPrices[3]  = 0;  // slaves
            this.$expectedPrices[6]  = 0;  // narcotics
            this.$expectedPrices[10] = 0;  // firearms
//		 this.$startMessage[0] = expandMissionText("autotrade_ethical_start");
            this.$startMessage[0] = expandMissionText("autotrade_ethical_start", {autotrade_countdown: (this.$firstTrade - this.$firstWarning).toFixed()});
            this.$mode[2] = 1;
            break;
          case 3:
            this.$expectedPrices[3]  = 152; // slaves
            this.$expectedPrices[6]  = 510; // narcotics
            this.$expectedPrices[10] = 692; // firearms
//		 this.$startMessage[0] = expandMissionText("autotrade_ruthless_start");
            this.$startMessage[0] = expandMissionText("autotrade_ruthless_start", {autotrade_countdown: (this.$firstTrade - this.$firstWarning).toFixed()});
            this.$mode[2] = 3;
            break;
          case 4:
            this.playerBoughtCargo = this.playerBoughtCargo_UNSET;
            this.playerSoldCargo = this.playerSoldCargo_UNSET;
//          this.$startMessage[0] = expandMissionText("autotrade_learning_start");
            this.$startMessage[0] = expandMissionText("autotrade_learning_start", {autotrade_countdown: (this.$firstTrade - this.$firstWarning).toFixed()});
            this.$mode[2] = 4;
            break;
          case 2:
          default:
            this.shipDockedWithStation = this.shipDockedWithStation_UNSET;
            if (player.ship.docked) {
                this.shipDockedWithStation(player.ship.dockedStation);
            }
            this.$mode[2] = 2;
            break;
        }
    }

/* ====================================================================================
			LEARNING BY OBSERVATION
======================================================================================= */

    this.playerBoughtCargo_UNSET = function (cargo, quantity, unitPrice) {
        "use strict";
        var ledgerNumber = this.$commodities.indexOf(cargo);
        if (unitPrice > this.$mode[3].buyBelow[ledgerNumber]) {
            this.$mode[3].buyBelow[ledgerNumber] = unitPrice;
            if (unitPrice > this.$mode[3].sellAbove[ledgerNumber]) {
                this.$mode[3].sellAbove[ledgerNumber] = unitPrice;
            }
        }
    }

    this.playerSoldCargo_UNSET = function (cargo, quantity, unitPrice) {
        "use strict";
        var ledgerNumber = this.$commodities.indexOf(cargo);
        if (unitPrice < this.$mode[3].sellAbove[ledgerNumber]) {
            this.$mode[3].sellAbove[ledgerNumber] = unitPrice;
            if (unitPrice < this.$mode[3].buyBelow[ledgerNumber]) {
                this.$mode[3].buyBelow[ledgerNumber] = unitPrice;
            }
        }
    }

/* ====================================================================================
			STARTING AND STOPPING AUTOTRADE
======================================================================================= */

    this._startAutotrade = function () {
        "use strict";
        this.$flummoxCounter = 0;
        switch (this.$mode[0]) {
          case "off":
            return false;
          case "buy":
            this.$adviceTimer = new Timer(this, function () {this._performTrading("advise-purchase", this.$mode[1]);}, this.$firstAdvice, this.$tradePeriod);
            break;
          case "sell":
            this.$adviceTimer = new Timer(this, function () {this._performTrading("advise-sale", this.$mode[1]);}, this.$firstAdvice, this.$tradePeriod);
            break;
          default:
            this.$adviceTimer = new Timer(this, function () {this._performTrading("advice", this.$mode[1]);}, this.$firstAdvice, this.$tradePeriod);
        }
        if (this.$mode[0] !== "advice") {
            this.$tradingTimer = new Timer(this, function () {this._performTrading(this.$mode[0], this.$mode[1]);}, this.$firstTrade, this.$tradePeriod);
            this.$warningTimer = new Timer(this, function () {this._startingActions();}, this.$firstWarning);
        }
        return true;
    }

    this._startingActions = function (silent) {
        "use strict";
        if (worldScripts['AI Trading Assistant']) {
            if (this.$mode[2] !== 4) { // Not in training mode.
                worldScripts['AI Trading Assistant']._setDisplayMode(4, false);
            }
            if (!silent) {
                worldScripts['AI Trading Assistant'].$showMessage(this.$startMessage[0], this.$startMessage[1], this.$startMessage[2]);
            }
        } else {
            player.commsMessage(this.$startMessage[0]);
        }
    }

    this._pauseAutotrade = function (pause) {
        "use strict";
        if (!pause || pause < 1) {
            pause = 8;
        }
        var earliest = clock.absoluteSeconds + pause;
        var timer = [this.$tradingTimer, this.$adviceTimer, this.$warningTimer];
        var addPause = false;
        for (var i = 0; i < timer.length; i++) {
            if (timer[i] && timer[i].isRunning && timer[i].nextTime < earliest) {
                addPause = true;
                break;
            }
        }
        if (addPause) {
            for (var i = 0; i < timer.length; i++) {
                if (timer[i] && timer[i].isRunning) {
                    timer[i].stop();
                    timer[i].nextTime += pause;
                    timer[i].start();
                }
            }
        }
        return pause;
    }

    this._stopAutotrade = function () {
        "use strict";
        if (this.$tradingTimer) {
            this.$tradingTimer.stop();
            delete this.$tradingTimer;
        }
        if (this.$adviceTimer) {
            this.$adviceTimer.stop();
            delete this.$adviceTimer;
        }
        if (this.$warningTimer) {
            this.$warningTimer.stop();
            delete this.$warningTimer;
        }
        delete this.$affectedScripts;
        delete this.$latestAdvice;
    }

    this.shipWillLaunchFromStation = function () {
        "use strict";
        this._stopAutotrade();
    }

/* ====================================================================================
			RESPONSES WHEN NO USEFUL TRADES ARE FOUND
======================================================================================= */

    this._flummoxResponse = function (flummoxCount) {
        "use strict";
        if (flummoxCount > 4) {
            this._stopAutotrade();
            worldScripts["AI Trading Assistant"].$showMessage(this.$finishMessage[0], this.$finishMessage[1], this.$finishMessage[2]);
        } else if (flummoxCount > 2) {
            if (this.$adviceTimer) {
                this.$adviceTimer.stop();
            }
            worldScripts["AI Trading Assistant"]._undoDisplayMode(4);
            worldScripts["AI Trading Assistant"].$showMessage(this.$noMoreMessage[0], this.$noMoreMessage[1], this.$noMoreMessage[2]);
        } else {
            worldScripts["AI Trading Assistant"].$showMessage(this.$flummoxMessage[0], this.$flummoxMessage[1], this.$flummoxMessage[2]);
        }
    }

    this._flummoxAlternative = function (flummoxCount) {
        "use strict";
        if (flummoxCount > 3) {
            this._stopAutotrade();
            player.commsMessage(this.$finishMessage[0]);
        }
    }

/* ====================================================================================
			PERFORM TRADING

	If hold has space, check for anything worth buying (will include precious commodities).
	If hold has no space, check for precious commodities worth buying.
	If cannot find anything to buy, check for anything worth selling.
======================================================================================= */

    this._performTrading = function (mode, reserveSpace) {
        "use strict";
        if (typeof reserveSpace === "undefined" && this.$mode && this.$mode[1]) {
            reserveSpace = this.$mode[1];
        }
        if (typeof reserveSpace === "undefined") {
            log(this.name, "ERROR: performTrading function could not find a reserve space parameter.");
            return false;
        }
        if (typeof mode === "undefined" && this.$mode && this.$mode[0]) {
            mode = this.$mode[0];
        }
        if (typeof mode === "undefined") {
            log(this.name, "ERROR: performTrading function could not find a mode parameter.");
            return false;
        }
        var bestDiscount = new Array(3);
        var bestPremium = new Array(4);
        var cargo;
        var quantity;
        var discount;
        var premium;
        var actualPremium;
        var maxPurchase = player.ship.cargoSpaceAvailable - reserveSpace;

     // Find something to buy - no profit to be made from an empty hold.

        if (mode !== "sell" && mode !== "advise-sale") {
        //  if (maxPurchase > 0) {
            if (player.ship.cargoSpaceAvailable > reserveSpace) {
                bestDiscount = this._findBestDiscount();
            } else if (player.credits > player.ship.cargoSpaceCapacity * 102) {
                bestDiscount = this._findDiscountedValuables();
            } else {
                bestDiscount = ["goods", 0, -1020];
            }
            cargo    = bestDiscount[0];
            quantity = bestDiscount[1];
            discount = bestDiscount[2];
            if (cargo !== "goods" && discount > 0) {
                var success = false;
                switch (mode) {
                  case "advise-purchase":
                  case "advice":
                    this.$latestAdvice = "buy " + cargo;
                    success = this._recommendPurchase(cargo);
                    break;
                  case "buy":
                  case "fast":
                    if (this.$latestAdvice === "buy " + cargo) {
                        success = this._buyAsMuchAsPossible(cargo, maxPurchase);
                    }
                    break;
                  case "slow":
                    if (this.$latestAdvice === "buy " + cargo) {
                        success = this._buyOneUnit(cargo);
                    }
                    break;
                  case "off":
                  default:
                    log(this.name, "ERROR: performTrading function was called with an unrecognised mode.");
                    return false;
                }
                if (success) {
                    this.$flummoxCounter = 0;
                    return true;
                } else {
                    this.$latestAdvice = "no money";
                    this.$flummoxCounter++;
                    this._flummoxResponse(this.$flummoxCounter);
                    return false;
                }
            }
        }

     // If can't find anything to buy, try to find something to sell...

        if (mode !== "buy" && mode !== "advise-purchase") {
            bestPremium = this._findBestPremium();
            cargo         = bestPremium[0];
            quantity      = bestPremium[1];
            premium       = bestPremium[2];
            actualPremium = bestPremium[3];
            if (cargo !== "goods" && premium > 0 && actualPremium > 0) {
                switch (mode) {
                  case "advise-sale":
                  case "advice":
                    this.$latestAdvice = "sell " + cargo;
                    this._recommendSale(cargo, actualPremium);
                    break;
                  case "sell":
                  case "fast":
                    if (this.$latestAdvice === "sell " + cargo) {
                        this._sellAsMuchAsPossible(cargo);
                    }
                    break;
                  case "slow":
                    if (this.$latestAdvice === "sell " + cargo) {
                        this._sellOneUnit(cargo);
                    }
                    break;
                  case "off":
                  default:
                    log(this.name, "ERROR: performTrading function was called with an unrecognised mode.");
                    return false;
                }
                this.$flummoxCounter = 0;
                return true;
            }
        }

     // Can't find a useful trade...

        this.$latestAdvice = "no trade";
        this.$flummoxCounter++;
        this._flummoxResponse(this.$flummoxCounter);
        return false;
    }

/* ====================================================================================
			FUNCTIONS TO FIND DISCOUNTS & PREMIUMS

	_findDiscountedValuables is an 'orrible kludge, requiring elegantization.
======================================================================================= */

    this._findBestDiscount = function () {
        "use strict";
        var bestDiscount = ["goods",0,-1020]; // [0] commodity, [1] quantity, [2] discount to expected price
        var market = player.ship.dockedStation.market;
        var commodity;
        var quantity;
        var discount;
        var arrayLength = this.$commodities.length;
        for (var i = 0; i < arrayLength; i++) {
            commodity = this.$commodities[i];
            quantity = market[commodity].quantity;
            if (quantity > 0) {
                if (this.$mode[2] === 4) {
                    discount = this.$mode[3].buyBelow[i] - market[commodity].price;
                } else {
                    discount = this.$expectedPrices[i] - market[commodity].price;
                }
                if (discount > bestDiscount[2]) {
                    bestDiscount[0] = commodity;
                    bestDiscount[1] = quantity;
                    bestDiscount[2] = discount;
                }
            }
        }
        return bestDiscount;
    }

    this._findDiscountedValuables = function () {
        "use strict";
        var bestDiscount = ["goods", 0, -1020]; // [0] commodity, [1] quantity, [2] discount to expected price
        var market = player.ship.dockedStation.market;
        var commodities = ["gold", "platinum", "gem_stones"];
        var commodity;
        var quantity;
        var discount;
        var arrayLength = commodities.length;
        for (var i = 0; i < arrayLength; i++) {
            commodity = commodities[i];
            quantity = market[commodity].quantity;
            if (quantity > 0) {
                if (this.$mode[2] === 4) {
                    discount = this.$mode[3].buyBelow[i + 13] - market[commodity].price;
                } else {
                    discount = this.$expectedPrices[i + 13] - market[commodity].price;
                }
                if (discount > bestDiscount[2]) {
                    bestDiscount[0] = commodity;
                    bestDiscount[1] = quantity;
                    bestDiscount[2] = discount;
                }
            }
        }
        return bestDiscount;
    }

   this._findBestPremium = function () {
        "use strict";
        var bestPremium = ["goods",0,-1020,-102]; // [0] commodity, [1] quantity, [2] premium over expected price, [3] premium over average price paid.
        var market = player.ship.dockedStation.market;
        var commodity;
        var quantity;
        var premium;
        var actualPremium;
        var arrayLength = this.$commodities.length;
        for (var i = 0; i < arrayLength; i++) {
            commodity = this.$commodities[i];
            quantity = manifest[commodity];
            if (quantity > 0 && market[commodity].quantity < 127) {
                if (this.$mode[2] === 4) {
                    premium = market[commodity].price - this.$mode[3].sellAbove[i];
                } else {
                    premium = market[commodity].price - this.$expectedPrices[i];
                }
                if (premium > bestPremium[2]) {
                    bestPremium[0] = commodity;
                    bestPremium[1] = quantity;
                    bestPremium[2] = premium;
                    if (worldScripts['AI Trading Assistant'] &&
                        worldScripts['AI Trading Assistant'].$averagesPaid) {
                        actualPremium = market[commodity].price / 10 - worldScripts['AI Trading Assistant'].$averagesPaid[i];
                    } else {
                        actualPremium = 0.1;
                    }
                    bestPremium[3] = actualPremium;
                }
            }
        }
        return bestPremium;
    }

/* ====================================================================================
			BUY ONE UNIT OF COMMODITY
======================================================================================= */

    this._buyOneUnit = function (cargo) {
        "use strict";
        var station = player.ship.dockedStation;
        var price = station.market[cargo].price;
        var quantity = station.market[cargo].quantity;
        if (price < player.credits * 10 &&
            quantity > 0 &&
            (player.ship.cargoSpaceAvailable > 0 || this._hasSpaceFor(cargo, 1))) {
            player.credits -= price / 10;
            manifest[cargo] += 1;
            station.setMarketQuantity(cargo, quantity - 1);
            this._broadcastEvent("bought", cargo, 1, price);
            return true;
        } else {
            return false;
        }
    }

/* ====================================================================================
			BUY AS MUCH OF COMMODITY AS POSSIBLE
======================================================================================= */

    this._buyAsMuchAsPossible = function (cargo, maxQuantity) {
        "use strict";
        var station      = player.ship.dockedStation;
        var price        = station.market[cargo].price;
        var quantity     = station.market[cargo].quantity;
        var quantityUnit = station.market[cargo].quantityUnit;
        var saleQuantity;
        if (maxQuantity < 1 && quantityUnit > 0) {
            maxQuantity = 127;
        }
        if (player.credits * 10 < maxQuantity * price) {
            maxQuantity = Math.floor(player.credits * 10 / price);
        }
        if (quantity > 0 &&
            maxQuantity > 0 &&
            (maxQuantity <= player.ship.cargoSpaceAvailable ||
            this._hasSpaceFor(cargo, maxQuantity))) {
            if (maxQuantity >= quantity) {
                saleQuantity = quantity;
            } else if (maxQuantity < quantity) {
                saleQuantity = maxQuantity;
            }
            player.credits -= saleQuantity * price / 10;
            manifest[cargo] += saleQuantity;
            station.setMarketQuantity(cargo, quantity - saleQuantity);
            this._broadcastEvent("bought", cargo, saleQuantity, price);
            return saleQuantity;
        } else {
            return false;
        }
    }

/* ====================================================================================
			CHECK PLAYER'S SHIP HAS SPACE FOR CARGO

	This is based on, but not identical to, ["oolite-contracts-cargo"]._hasSpaceFor
======================================================================================= */

    this._hasSpaceFor = function (cargo, quantity) {
        "use strict";
        var amountInTC = quantity;
        if (system.mainStation.market[cargo].quantityUnit == "1") {
            var spareSafe = 499 - player.ship.manifest[cargo] % 1000;
            amountInTC -= spareSafe;
            amountInTC = Math.ceil(amountInTC / 1000);
            if (amountInTC < 0) {
                amountInTC = 0;
            }
        } else if (system.mainStation.market[cargo].quantityUnit == "2") {
            var spareSafe = 499999 - player.ship.manifest[cargo] % 1000000;
            amountInTC -= spareSafe;
            amountInTC = Math.ceil(amountInTC / 1000000);
            if (amountInTC < 0) {
                amountInTC = 0;
            }
        }
        return (amountInTC <= player.ship.cargoSpaceAvailable);
    }

/* ====================================================================================
			SELL ONE UNIT OF COMMODITY
======================================================================================= */

   this._sellOneUnit = function (cargo) {
        "use strict";
        var station  = player.ship.dockedStation;
        var price    = station.market[cargo].price;
        var quantity = station.market[cargo].quantity;
        if (quantity < 127 && manifest[cargo] > 0) {
            manifest[cargo] -= 1;
            player.credits += price / 10;
            station.setMarketQuantity(cargo, quantity + 1);
            this._broadcastEvent("sold", cargo, 1, price);
            return true;
        } else {
            return false;
        }
    }

/* ====================================================================================
			SELL AS MUCH OF COMMODITY AS POSSIBLE
======================================================================================= */

    this._sellAsMuchAsPossible = function (cargo) {
        "use strict";
        var station     = player.ship.dockedStation;
        var price       = station.market[cargo].price;
        var quantity    = station.market[cargo].quantity;
        var maxQuantity = 127 - quantity;
        var saleQuantity;
        if (quantity < 127 && manifest[cargo] > 0) {
            if (maxQuantity >= manifest[cargo]) {
                saleQuantity = manifest[cargo];
            } else if (maxQuantity < manifest[cargo]) {
                saleQuantity = maxQuantity;
            }
            manifest[cargo] -= saleQuantity;
            player.credits += saleQuantity * price / 10;
            station.setMarketQuantity(cargo, quantity + saleQuantity);
            this._broadcastEvent("sold", cargo, saleQuantity, price);
            return saleQuantity;
        } else {
            return false;
        }
    }

/* ====================================================================================
			RECOMMEND COMMODITY TO BUY
======================================================================================= */

    this._recommendPurchase = function (cargo) {
        "use strict";
        var market = player.ship.dockedStation.market;
        var price = market[cargo].price;
        var quantity = market[cargo].quantity;
        var displayName = market[cargo].displayName;
        var message = expandMissionText("autotrade_purchase_recommendation", {autotrade_Commodity: displayName, autotrade_commodity: displayName.toLowerCase()});
        if (price < player.credits * 10 &&
            quantity > 0 &&
            (player.ship.cargoSpaceAvailable > 0 || this._hasSpaceFor(cargo, 1))) {
            if (worldScripts['AI Trading Assistant'] &&
                worldScripts['AI Trading Assistant'].$tradingMessages) {
                worldScripts['AI Trading Assistant'].$showMessage(message, "", cargo);
            } else {
                player.commsMessage(message);
            }
            return true;
        } else {
            return false;
        }
    }

/* ====================================================================================
			RECOMMEND COMMODITY TO SELL
======================================================================================= */

    this._recommendSale = function (cargo, actualPremium) {
        "use strict";
        var market = player.ship.dockedStation.market;
        var quantity = market[cargo].quantity;
        var displayName = market[cargo].displayName;
        var message = expandMissionText("autotrade_sale_recommendation", {autotrade_Commodity: displayName, autotrade_commodity: displayName.toLowerCase()});
        if (actualPremium &&
            this.$mode[0] !== "slow") {
            message += expandMissionText("autotrade_sale_actual_premium", {autotrade_actual_premium: actualPremium.toFixed(1)});
        }
        if (quantity < 127 && manifest[cargo] > 0) {
            if (worldScripts['AI Trading Assistant'] &&
                worldScripts['AI Trading Assistant'].$tradingMessages) {
                worldScripts['AI Trading Assistant'].$showMessage(message, "", cargo);
            } else {
                player.commsMessage(message);
            }
            return true;
        } else {
            return false;
        }
    }

/* ====================================================================================
			NOTIFY OTHER SCRIPTS ABOUT TRADES PERFORMED

	$affectedScripts is compiled automatically when first needed, and includes all 
	scripts which listen for playerBoughtCargo or playerSoldCargo. I'm in two minds 
	about the need to, and when best to, delete it so it will be re-compiled.
======================================================================================= */

    this._broadcastEvent = function (event, cargo, quantity, price) {
        "use strict";
        if (!this.$affectedScripts) {
            this._makeBroadcastList();
        }
        var scriptName;
        var arrayLength = this.$affectedScripts.length;
        for (var i = 0; i < arrayLength; i++) {
            scriptName = this.$affectedScripts[i];
            if (worldScripts[scriptName]) {
                if (event === "bought" &&
                    worldScripts[scriptName].playerBoughtCargo) {
                    worldScripts[scriptName].playerBoughtCargo(cargo, quantity, price);
                } else if (event === "sold" &&
                    worldScripts[scriptName].playerSoldCargo) {
                    worldScripts[scriptName].playerSoldCargo(cargo, quantity, price);
                }
            }
        }
        var sound = new SoundSource;
        if (event === "bought") {
            sound.sound = "[buy-commodity]";
        } else if (event === "sold") {
            sound.sound = "[sell-commodity]";
        }
        sound.play();
    }

    this._makeBroadcastList = function () {
        "use strict";
        this.$affectedScripts = [];
        var scriptName;
        var arrayLength = worldScriptNames.length;
        for (var i = 0; i < arrayLength; i++) {
            scriptName = worldScriptNames[i];
            if (scriptName !== this.name &&
                (worldScripts[scriptName].playerBoughtCargo ||
                worldScripts[scriptName].playerSoldCargo)) {
                this.$affectedScripts.push(scriptName);
            }
        }
    }

    this.guiScreenChanged = function (to, from) {
        "use strict";
        if (from === "GUI_SCREEN_MARKET") {
            delete this.$affectedScripts;
        }
    }

/* ====================================================================================
			COPYRIGHT NOTICE

	(C)2013-2014, Wildeblood. All rights reserved. 
	Free to use, but DO NOT PROMULGATE MODIFIED VERSIONS.

This was created by some fellow with the user name "Wildeblood" on the Oolite forums, 
he'll tell you his real name if you bother to ask.
To use means to play with, modify or re-use in whole or in part, and to distribute freely.
To use DOES NOT imply to use for commercial gain, which would be a breach of copyright.
To promulgate means to make official announcements.
======================================================================================= */
