"use strict";

this.name    = "AI Buying Advisor"; // Messages displayed when player BUYS cargo.
this.version = "2.12";              // No changes since version 2.2.6.

/* ====================================================================================
			CONGRATULATIONS DISPLAYED IF CARGO IS BOUGHT FOR LOW PRICES
======================================================================================= */

this.$paidLowPrice = function(cargo)
	{
	this.$message = "Nice one, boss, you bought that for a good, low price.";
	this.$summary = "Nice one!";
	}

this.$paidBelowGalCopPrice = function(cargo)
	{
	this.$message = "Excellent, boss, you’d never buy it that cheaply at a GalCop station.";
	this.$summary = "Excellent!";
	}

/* ====================================================================================
			WARNINGS DISPLAYED IF CARGO IS BOUGHT FOR HIGH PRICES
======================================================================================= */

this.$paidAboveAveragePrice = function(cargo)
	{
	this.$message   = "Whoa, boss, you’re paying above the average price.";
	this.$summary   = "Alert!";
	this.$highlight = cargo;
	}

this.$paidFarTooMuch = function(cargo)
	{
	var displayName = displayNameForCommodity(cargo);
	this.$message   = "Whoa, boss, that’s a silly price to pay for " + displayName.toLowerCase() + ".";
	this.$summary   = "Warning!";
	this.$highlight = cargo;
	}

/* ====================================================================================
			WARNINGS DISPLAYED IF ILLEGAL GOODS ARE BOUGHT (UNLESS PRICE IS LOW)
======================================================================================= */

this.$illegalGoods = function(cargo)
	{
	var displayName = displayNameForCommodity(cargo);
	if (player.ship.dockedStation.isMainStation)
		{
		this.$message = "Whoa, boss! It’s illegal to leave this station with " + displayName.toLowerCase() + " aboard.";
		this.$summary = "Illegal!";
		this.$highlight = cargo;
		}
	else
		{
		this.$message = "Whoa, boss, you don’t want " + displayName.toLowerCase() + " on board.";
		this.$summary = "Hmm.";
		}
	}

/* ====================================================================================
			WARNING DISPLAYED IF DESTINATION SYSTEM HAS SIMILAR ECONOMY
======================================================================================= */

this.$similarDestination = function(cargo, destination)
	{
	var displayName = displayNameForCommodity(cargo);
	if (player.ship.dockedStation.isMainStation)
		{
		this.$message = "You’re not likely to make much profit selling " + displayName.toLowerCase() + " at " + destination.name + ", boss.";
		}
	}

/* ====================================================================================
			CASES FOR PLAYER BUYING CARGO
======================================================================================= */

this.$reactToPlayerBuyingCargo = function(cargo, quantity, price)
	{
	this.$message   = "";
	this.$summary   = "Buying.";
	this.$highlight = "";

	var similarDestination = false;
	if (player.ship.targetSystem != system.ID) // Destination is set, so check it out...
		{
		var destination = System.infoForSystem(galaxyNumber, player.ship.targetSystem);
		if ((system.economy > 3 && destination.economy > 3)
		 || (system.economy < 4 && destination.economy < 4))
			{
			similarDestination = true;
			}
		}

	price = price/10;
	switch(cargo)
		{
		case "food":
			if      (price > 6.5)  this.$paidFarTooMuch(cargo); // Maximum GalCop price: 8.0
			else if (price > 5.0)  this.$paidAboveAveragePrice(cargo);  // Paying above the average price.
			if      (price < 2.0)  this.$paidBelowGalCopPrice(cargo); // Minimum GalCop price: 2.0
			else if (price < 3.5)  this.$paidLowPrice(cargo); // Minimum GalCop price: 2.0
			break;

		case "textiles":
			if      (price > 8.2)  this.$paidFarTooMuch(cargo); // Maximum GalCop price: 9.2
			else if (price > 7.2)  this.$paidAboveAveragePrice(cargo);
			if      (price < 5.2)  this.$paidBelowGalCopPrice(cargo); // Minimum GalCop price: 5.2
			else if (price < 6.2)  this.$paidLowPrice(cargo); // Minimum GalCop price: 5.2
			break;

		case "radioactives":
			if      (price > 26.0) this.$paidFarTooMuch(cargo); // Maximum GalCop price: 28.8
			else if (price > 23.2) this.$paidAboveAveragePrice(cargo);
			if      (price < 17.6) this.$paidBelowGalCopPrice(cargo); // Minimum GalCop price: 17.6
			else if (price < 20.4) this.$paidLowPrice(cargo); // Minimum GalCop price: 17.6
			break;

		case "slaves":
		//	if      (price > 15.2) this.$paidAboveAveragePrice(cargo);
			if      (price < 2.0)  this.$paidBelowGalCopPrice(cargo); // Minimum GalCop price: 2.0
			else if (price < 8.6)  this.$paidLowPrice(cargo); // Minimum GalCop price: 2.0
			else                   this.$illegalGoods(cargo);
			break;

		case "liquor_wines":
			if      (price > 34.2) this.$paidFarTooMuch(cargo); // Maximum GalCop price: 39.2
			else if (price > 29.2) this.$paidAboveAveragePrice(cargo);
			if      (price < 19.2) this.$paidBelowGalCopPrice(cargo); // Minimum GalCop price: 19.2
			else if (price < 24.2) this.$paidLowPrice(cargo); // Minimum GalCop price: 19.2
			else if (price < 26.7)
				{
				this.$message = "I’d pay that much for lethal brandy, but not for evil juice.";
				this.$summary = "Hmm.";
				}
			break;

		case "luxuries":
			if      (price > 96.1) this.$paidFarTooMuch(cargo); // Maximum GalCop price: 102.0
			else if (price > 90.2) this.$paidAboveAveragePrice(cargo);
			if      (price < 78.0) this.$paidBelowGalCopPrice(cargo); // Minimum GalCop price: 78.0
			else if (price < 84.1) this.$paidLowPrice(cargo); // Minimum GalCop price: 78.0
			break;

		case "narcotics":
		//	if      (price > 51.0) this.$paidAboveAveragePrice(cargo);
			if      (price < 0.4)  this.$paidBelowGalCopPrice(cargo); // Minimum GalCop price: 0.4
			else if (price < 25.7) this.$paidLowPrice(cargo); // Minimum GalCop price: 0.4
			else                   this.$illegalGoods(cargo);
			break;

		case "computers":
			if      (price > 91.9) this.$paidFarTooMuch(cargo); // Maximum GalCop price: 102.0
			else if (price > 81.8) this.$paidAboveAveragePrice(cargo);
			if      (price < 61.6) this.$paidBelowGalCopPrice(cargo); // Minimum GalCop price: 61.6
			else if (price < 71.7) this.$paidLowPrice(cargo); // Minimum GalCop price: 61.6
			else if (price < 81.8
			     && similarDestination) this.$similarDestination(cargo, destination); // Similar destination
			break;

		case "machinery":
			if      (price > 61.5) this.$paidFarTooMuch(cargo); // Maximum GalCop price: 66.4
			else if (price > 56.6) this.$paidAboveAveragePrice(cargo);
			if      (price < 46.8) this.$paidBelowGalCopPrice(cargo); // Minimum GalCop price: 46.8
			else if (price < 51.7) this.$paidLowPrice(cargo); // Minimum GalCop price: 46.8
			break;

		case "alloys":
			if      (price > 42.6) this.$paidFarTooMuch(cargo); // Maximum GalCop price: 46.4
			else if (price > 38.8) this.$paidAboveAveragePrice(cargo);
			if      (price < 31.2) this.$paidBelowGalCopPrice(cargo); // Minimum GalCop price: 31.2
			else if (price < 35.0) this.$paidLowPrice(cargo); // Minimum GalCop price: 31.2
			break;

		case "firearms":
		//	if      (price > 69.2) this.$paidAboveAveragePrice(cargo);
			if      (price < 49.6) this.$paidBelowGalCopPrice(cargo); // Minimum GalCop price: 49.6
			else if (price < 59.4) this.$paidLowPrice(cargo); // Minimum GalCop price: 49.6
			else                   this.$illegalGoods(cargo);
			break;

		case "furs":
			if      (price > 83.0) this.$paidFarTooMuch(cargo); // Maximum GalCop price: 95.6
			else if (price > 70.4) this.$paidAboveAveragePrice(cargo);
			if      (price < 45.2) this.$paidBelowGalCopPrice(cargo); // Minimum GalCop price: 45.2
			else if (price < 57.8) this.$paidLowPrice(cargo); // Minimum GalCop price: 45.2
			else if (price < 70.4
			     && similarDestination) this.$similarDestination(cargo, destination); // Similar destination
			break;

		case "minerals":
			if      (price > 13.0) this.$paidFarTooMuch(cargo); // Maximum GalCop price: 14.0
			else if (price > 12.0) this.$paidAboveAveragePrice(cargo);
			if      (price < 10.0) this.$paidBelowGalCopPrice(cargo); // Minimum GalCop price: 10.0
			else if (price < 11.0) this.$paidLowPrice(cargo); // Minimum GalCop price: 10.0
			break;

		case "gold":
			if      (price > 40.2) this.$paidFarTooMuch(cargo); // Maximum GalCop price: 41.6
			else if (price > 38.8) this.$paidAboveAveragePrice(cargo);
			if      (price < 36.0) this.$paidBelowGalCopPrice(cargo); // Below minimum GalCop price: 36.0
			else if (price < 37.4) this.$paidLowPrice(cargo); // Minimum GalCop price: 36.0
			break;

		case "platinum":
			if      (price > 76.3) this.$paidFarTooMuch(cargo); // Maximum GalCop price: 80.8
			else if (price > 71.8) this.$paidAboveAveragePrice(cargo);
			if      (price < 62.8) this.$paidBelowGalCopPrice(cargo); // Below minimum GalCop price: 62.8
			else if (price < 67.3) this.$paidLowPrice(cargo); // Minimum GalCop price: 62.8
			break;

		case "gem_stones":
			if      (price > 21.8) this.$paidFarTooMuch(cargo); // Maximum GalCop price: 24.0
			else if (price > 19.6) this.$paidAboveAveragePrice(cargo);
			if      (price < 15.2) this.$paidBelowGalCopPrice(cargo); // Below minimum GalCop price: 15.2
			else if (price < 17.4) this.$paidLowPrice(cargo); // Minimum GalCop price: 15.2
			break;

		case "alien_items":
			if      (price > 54.8) this.$paidFarTooMuch(cargo); // Maximum GalCop price: 66.0
			else if (price > 43.6) this.$paidAboveAveragePrice(cargo);
			if      (price < 21.2) this.$paidBelowGalCopPrice(cargo); // Below minimum GalCop buying price: 21.2
			else if (price < 32.4) this.$paidLowPrice(cargo); // Minimum GalCop (buying) price: 21.2
			break;
		}
	worldScripts["AI Trading Assistant"].$showMessage(this.$message, this.$summary, this.$highlight);
	}

/* ====================================================================================
			SHOW MESSAGE - THS IS THE FUNCTION OTHER SCRIPTS SHOULD CALL

worldScripts["AI Trading Assistant"].$showMessage(message, summary, highlight);

message   - is the comms message to appear in the lower left of the screen.
summary   - is the one-word message that may appear at the bottom of the costs column
            and be read out by the text-to-speech function.
highlight - is the commodity type to be highlighted by a HUD switch.

======================================================================================= */